@@ -1,5 +1,5 @@
 #############################################################################
-#   Copyright (c) 2009 Marie Laure Delignette-Muller, Regis Pouillot, Jean-Baptiste Denis, Christophe Dutang                                                                                                  
+#   Copyright (c) 2009 Marie Laure Delignette-Muller                                                                                                  
 #                                                                                                                                                                        
 #   This program is free software; you can redistribute it and/or modify                                               
 #   it under the terms of the GNU General Public License as published by                                         
@@ -117,11 +117,13 @@ gofstat <- function (f, chisqbreaks, meancount, print.test = FALSE)
         # ad <-  -n-sum((2*(1:n)-1)*log(theop) + (2*n+1-2*(1:n))*log(1-theop))/n 
         if (is.null(fix.arg) & f$method == "mle")
         {
-          if ((distname == "norm" | distname == "lnorm") & n>=5) {
-            a2mod <- ad*(1+0.75/n+2.25/n^2)
-            adtest <- ifelse(a2mod>0.752,"rejected","not rejected")
-          } 
-          else
+            # the following test does not correspond to MLE estimate but to unbiased 
+            # estimate of the variance
+          #if ((distname == "norm" | distname == "lnorm") & n>=5) {
+          #  a2mod <- ad*(1+0.75/n+2.25/n^2)
+          #  adtest <- ifelse(a2mod>0.752,"rejected","not rejected")
+          #} 
+          #else
             if (distname == "exp" & n>=5) {
                 a2mod <- ad*(1+0.6/n)
                 adtest <- ifelse(a2mod>1.321,"rejected","not rejected")
@@ -140,11 +142,12 @@ gofstat <- function (f, chisqbreaks, meancount, print.test = FALSE)
                         adtest <- ifelse(a2mod>0.757,"rejected","not rejected")
                     }
                     else
-                        if (distname == "logis" & n>=5) {
-                            a2mod <- ad*(1+0.25/n)
-                            adtest <- ifelse(a2mod>0.66,"rejected","not rejected")
-                        }
-                        else
+                        # the following test does not correspond to MLE estimate  
+                        # if (distname == "logis" & n>=5) {
+                        #    a2mod <- ad*(1+0.25/n)
+                        #    adtest <- ifelse(a2mod>0.66,"rejected","not rejected")
+                        #}
+                        # else
                             if (distname == "cauchy" & n>=5) {
                                 interp <- approxfun(c(5,8,10,12,15,20,25,30,40,50,60,100),
                                 c(1.77,3.2,3.77,4.14,4.25,4.05,3.57,3.09,2.48,2.14,1.92,1.52),
@@ -161,11 +164,13 @@ gofstat <- function (f, chisqbreaks, meancount, print.test = FALSE)
         
         if (is.null(fix.arg) & f$method == "mle")
         {
-          if ((distname == "norm" | distname == "lnorm") & n>=5) {
-            w2mod <- cvm*(1+0.5/n)
-            cvmtest <- ifelse(w2mod>0.126,"rejected","not rejected")
-          } 
-          else
+            # the following test does not correspond to MLE estimate but to unbiased 
+            # estimate of the variance
+          # if ((distname == "norm" | distname == "lnorm") & n>=5) {
+          #  w2mod <- cvm*(1+0.5/n)
+          #  cvmtest <- ifelse(w2mod>0.126,"rejected","not rejected")
+          # } 
+          # else
             if (distname == "exp" & n>=5) {
                 w2mod <- cvm*(1+0.16/n)
                 cvmtest <- ifelse(w2mod>0.222,"rejected","not rejected")
@@ -180,15 +185,16 @@ gofstat <- function (f, chisqbreaks, meancount, print.test = FALSE)
                 }
                 else
                     if (distname == "weibull" & n>=5) {
-                        w2mod <- ad*(1+0.2/sqrt(n))
+                        w2mod <- cvm*(1+0.2/sqrt(n))
                         cvmtest <- ifelse(w2mod>0.124,"rejected","not rejected")
                     }
                     else
-                        if (distname == "logis" & n>=5) {
-                            w2mod <- (n*cvm - 0.08)/(n - 1)
-                            cvmtest <- ifelse(w2mod>0.098,"rejected","not rejected")
-                        }
-                        else
+                        # the following test does not correspond to MLE estimate 
+                        # if (distname == "logis" & n>=5) {
+                        #     w2mod <- (n*cvm - 0.08)/(n - 1)
+                        #     cvmtest <- ifelse(w2mod>0.098,"rejected","not rejected")
+                        # }
+                        # else
                             if (distname == "cauchy" & n>=5) {
                                 interp <- approxfun(c(5,8,10,12,15,20,25,30,40,50,60,100),
                                 c(0.393,0.703,0.833,0.896,0.904,0.835,0.726,0.615,0.460,0.381,0.330,0.2378),
